package logs

import (
	"context"

	errorsgrpc "git.perx.ru/perxis/perxis-go/pkg/errors/grpc"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	pb "git.perx.ru/perxis/perxis-go/proto/logs"
	"google.golang.org/grpc"
)

type Client struct {
	client pb.LogsServiceClient
}

var _ Service = &Client{}

func NewClient(conn *grpc.ClientConn) *Client {
	return &Client{
		client: pb.NewLogsServiceClient(conn),
	}
}

func (c *Client) Log(ctx context.Context, entries []*Entry) error {
	var pbEntries []*pb.LogEntry
	for _, e := range entries {
		pbEntries = append(pbEntries, EntryToPB(e))
	}
	response, err := c.client.Log(ctx, &pb.LogRequest{Entries: pbEntries})
	if err != nil {
		return err
	}
	if response.GetError() != nil {
		return errorsgrpc.ErrorFromProto(nil, response.GetError())
	}
	return nil
}

func (c *Client) Find(ctx context.Context, filter *Filter, opts *options.FindOptions) (*FindResult, error) {
	request := new(pb.FindRequest)

	if filter != nil {
		request.Filter = &pb.Filter{Q: filter.Q}
	}

	request.Options = options.FindOptionsToPB(opts)

	response, err := c.client.Find(ctx, request)
	if err != nil {
		return nil, err
	}
	if response.GetError() != nil {
		return nil, errorsgrpc.ErrorFromProto(nil, response.GetError())
	}
	return FindResultFromPB(response.GetResult()), nil
}

func (c *Client) Delete(ctx context.Context, filter *Filter) error {
	request := new(pb.DeleteRequest)
	if filter != nil {
		request.Filter = &pb.Filter{Q: filter.Q}
	}
	response, err := c.client.Delete(ctx, &pb.DeleteRequest{Filter: &pb.Filter{Q: filter.Q}})
	if err != nil {
		return err
	}
	if response.GetError() != nil {
		return errorsgrpc.ErrorFromProto(nil, response.GetError())
	}
	return nil
}
